@@ -21,33 +21,44 @@
 #if !defined (_BASHHIST_H_)
 #define _BASHHIST_H_
 
+#include "stdc.h"
+
+/* Flag values for history_control */
+#define HC_IGNSPACE	0x01
+#define HC_IGNDUPS	0x02
+#define HC_ERASEDUPS	0x04
+
+#define HC_IGNBOTH	(HC_IGNSPACE|HC_IGNDUPS)
+
 extern int remember_on_history;
 extern int history_lines_this_session;
 extern int history_lines_in_file;
 extern int history_expansion;
 extern int history_control;
 extern int command_oriented_history;
+extern int current_command_first_line_saved;
 extern int hist_last_line_added;
 
 #  if defined (BANG_HISTORY)
 extern int history_expansion_inhibited;
 #  endif /* BANG_HISTORY */
 
-extern void bash_initialize_history ();
-extern void bash_history_reinit ();
-extern void bash_history_disable ();
-extern void bash_history_enable ();
-extern void load_history ();
-extern void save_history ();
-extern int maybe_append_history ();
-extern int maybe_save_shell_history ();
-extern char *pre_process_line ();
-extern int history_number ();
-extern void maybe_add_history ();
-extern void bash_add_history ();
+extern void bash_initialize_history __P((void));
+extern void bash_history_reinit __P((int));
+extern void bash_history_disable __P((void));
+extern void bash_history_enable __P((void));
+extern void load_history __P((void));
+extern void save_history __P((void));
+extern int maybe_append_history __P((char *));
+extern int maybe_save_shell_history __P((void));
+extern char *pre_process_line __P((char *, int, int));
+extern void maybe_add_history __P((char *));
+extern void bash_add_history __P((char *));
+extern int check_add_history __P((char *, int));
+extern int history_number __P((void));
 
-extern void setup_history_ignore ();
+extern void setup_history_ignore __P((char *));
 
-extern char *last_history_line ();
+extern char *last_history_line __P((void));
 
 #endif /* _BASHHIST_H_ */
