@@ -1,4 +1,4 @@
-/* Copyright (C) 1987, 1989, 1991 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2004 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -36,12 +36,11 @@
 #if defined (PREFER_STDARG)
 #  include <stdarg.h>
 #else
-#  if defined (PREFER_VARARGS)
-#    include <varargs.h>
-#  endif
+#  include <varargs.h>
 #endif
 
 #include "../bashansi.h"
+#include "../bashintl.h"
 
 #include "../shell.h"
 #include "maxpath.h"
@@ -64,8 +63,7 @@
 extern int errno;   
 #endif /* !errno */
 
-extern int no_symbolic_links;
-extern int indirection_level, startup_state, subshell_environment;
+extern int indirection_level, subshell_environment;
 extern int line_number;
 extern int last_command_exit_value;
 extern int running_trap;
@@ -86,7 +84,6 @@ sh_builtin_func_t *this_shell_builtin = (sh_builtin_func_t *)NULL;
 /* This is a lot like report_error (), but it is for shell builtins
    instead of shell control structures, and it won't ever exit the
    shell. */
-#if defined (USE_VARARGS)
 void
 #if defined (PREFER_STDARG)
 builtin_error (const char *format, ...)
@@ -102,32 +99,18 @@ builtin_error (format, va_alist)
   name = get_name_for_error ();
   fprintf (stderr, "%s: ", name);
 
+  if (interactive_shell == 0)
+    fprintf (stderr, "line %d: ", executing_line_number ());
+
   if (this_command_name && *this_command_name)
     fprintf (stderr, "%s: ", this_command_name);
 
-#if defined (PREFER_STDARG)
-  va_start (args, format);
-#else
-  va_start (args);
-#endif
+  SH_VA_START (args, format);
 
   vfprintf (stderr, format, args);
   va_end (args);
   fprintf (stderr, "\n");
 }
-#else /* !USE_VARARGS */
-void
-builtin_error (format, arg1, arg2, arg3, arg4, arg5)
-     char *format, *arg1, *arg2, *arg3, *arg4, *arg5;
-{
-  if (this_command_name && *this_command_name)
-    fprintf (stderr, "%s: ", this_command_name);
-
-  fprintf (stderr, format, arg1, arg2, arg3, arg4, arg5);
-  fprintf (stderr, "\n");
-  fflush (stderr);
-}
-#endif /* !USE_VARARGS */
 
 /* Print a usage summary for the currently-executing builtin command. */
 void
@@ -147,20 +130,11 @@ no_args (list)
 {
   if (list)
     {
-      builtin_error ("too many arguments");
+      builtin_error (_("too many arguments"));
       jump_to_top_level (DISCARD);
     }
 }
 
-/* Function called when one of the builtin commands detects a bad
-   option. */
-void
-bad_option (s)
-     char *s;
-{
-  builtin_error ("unknown option: %s", s);
-}
-
 /* Check that no options were given to the currently-executing builtin,
    and return 0 if there were options. */
 int
@@ -176,6 +150,126 @@ no_options (list)
   return (0);
 }
 
+void
+sh_needarg (s)
+     char *s;
+{
+  builtin_error (_("%s: option requires an argument"), s);
+}
+
+void
+sh_neednumarg (s)
+     char *s;
+{
+  builtin_error (_("%s: numeric argument required"), s);
+}
+
+void
+sh_notfound (s)
+     char *s;
+{
+  builtin_error (_("%s: not found"), s);
+}
+
+/* Function called when one of the builtin commands detects an invalid
+   option. */
+void
+sh_invalidopt (s)
+     char *s;
+{
+  builtin_error (_("%s: invalid option"), s);
+}
+
+void
+sh_invalidoptname (s)
+     char *s;
+{
+  builtin_error (_("%s: invalid option name"), s);
+}
+
+void
+sh_invalidid (s)
+     char *s;
+{
+  builtin_error (_("`%s': not a valid identifier"), s);
+}
+
+void
+sh_invalidnum (s)
+     char *s;
+{
+  builtin_error (_("%s: invalid number"), s);
+}
+
+void
+sh_invalidsig (s)
+     char *s;
+{
+  builtin_error (_("%s: invalid signal specification"), s);
+}
+
+void
+sh_badpid (s)
+     char *s;
+{
+  builtin_error (_("`%s': not a pid or valid job spec"), s);
+}
+
+void
+sh_readonly (s)
+     const char *s;
+{
+  builtin_error (_("%s: readonly variable"), s);
+}
+
+void
+sh_erange (s, desc)
+     char *s, *desc;
+{
+  if (s)
+    builtin_error (_("%s: %s out of range"), s, desc ? desc : _("argument"));
+  else
+    builtin_error (_("%s out of range"), desc ? desc : _("argument"));
+}
+
+#if defined (JOB_CONTROL)
+void
+sh_badjob (s)
+     char *s;
+{
+  builtin_error (_("%s: no such job"), s);
+}
+
+void
+sh_nojobs (s)
+     char *s;
+{
+  if (s)
+    builtin_error (_("%s: no job control"), s);
+  else
+    builtin_error (_("no job control"));
+}
+#endif
+
+#if defined (RESTRICTED_SHELL)
+void
+sh_restricted (s)
+     char *s;
+{
+  if (s)
+    builtin_error (_("%s: restricted"), s);
+  else
+    builtin_error (_("restricted"));
+}
+#endif
+
+void
+sh_notbuiltin (s)
+     char *s;
+{
+  builtin_error (_("%s: not a shell builtin"), s);
+}
+
 /* **************************************************************** */
 /*								    */
 /*	     Shell positional parameter manipulation		    */
@@ -192,7 +286,7 @@ make_builtin_argv (list, ip)
 {
   char **argv;
 
-  argv = word_list_to_argv (list, 0, 1, ip);
+  argv = strvec_from_word_list (list, 0, 1, ip);
   argv[0] = this_command_name;
   return argv;
 }
@@ -235,67 +329,6 @@ remember_args (list, destructive)
     set_dollar_vars_changed ();
 }
 
-/* **************************************************************** */
-/*								    */
-/*		 Pushing and Popping variable contexts		    */
-/*								    */
-/* **************************************************************** */
-
-static WORD_LIST **dollar_arg_stack = (WORD_LIST **)NULL;
-static int dollar_arg_stack_slots;
-static int dollar_arg_stack_index;
-
-void
-push_context ()
-{
-  push_dollar_vars ();
-  variable_context++;
-}
-
-void
-pop_context ()
-{
-  pop_dollar_vars ();
-  kill_all_local_variables ();
-  variable_context--;
-}
-
-/* Save the existing positional parameters on a stack. */
-void
-push_dollar_vars ()
-{
-  if (dollar_arg_stack_index + 2 > dollar_arg_stack_slots)
-    {
-      dollar_arg_stack = (WORD_LIST **)
-	xrealloc (dollar_arg_stack, (dollar_arg_stack_slots += 10)
-		  * sizeof (WORD_LIST **));
-    }
-  dollar_arg_stack[dollar_arg_stack_index++] = list_rest_of_args ();
-  dollar_arg_stack[dollar_arg_stack_index] = (WORD_LIST *)NULL;
-}
-
-/* Restore the positional parameters from our stack. */
-void
-pop_dollar_vars ()
-{
-  if (!dollar_arg_stack || dollar_arg_stack_index == 0)
-    return;
-
-  remember_args (dollar_arg_stack[--dollar_arg_stack_index], 1);
-  dispose_words (dollar_arg_stack[dollar_arg_stack_index]);
-  dollar_arg_stack[dollar_arg_stack_index] = (WORD_LIST *)NULL;
-}
-
-void
-dispose_saved_dollar_vars ()
-{
-  if (!dollar_arg_stack || dollar_arg_stack_index == 0)
-    return;
-
-  dispose_words (dollar_arg_stack[dollar_arg_stack_index]);
-  dollar_arg_stack[dollar_arg_stack_index] = (WORD_LIST *)NULL;
-}
-
 static int changed_dollar_vars;
 
 /* Have the dollar variables been reset to new values since we last
@@ -315,7 +348,12 @@ set_dollar_vars_unchanged ()
 void
 set_dollar_vars_changed ()
 {
-  changed_dollar_vars = 1;
+  if (variable_context)
+    changed_dollar_vars |= ARGS_FUNC;
+  else if (this_shell_builtin == set_builtin)
+    changed_dollar_vars |= ARGS_SETBLTIN;
+  else
+    changed_dollar_vars |= ARGS_INVOC;
 }
 
 /* **************************************************************** */
@@ -330,21 +368,24 @@ set_dollar_vars_changed ()
    follow.  If FATAL is true, call throw_to_top_level, which exits the
    shell; if not, call jump_to_top_level (DISCARD), which aborts the
    current command. */
-long
+intmax_t
 get_numeric_arg (list, fatal)
      WORD_LIST *list;
      int fatal;
 {
-  long count = 1;
+  intmax_t count = 1;
+
+  if (list && list->word && ISOPTION (list->word->word, '-'))
+    list = list->next;
 
   if (list)
     {
       register char *arg;
 
       arg = list->word->word;
-      if (!arg || (legal_number (arg, &count) == 0))
+      if (arg == 0 || (legal_number (arg, &count) == 0))
 	{
-	  builtin_error ("bad non-numeric arg `%s'", list->word->word);
+	  sh_neednumarg (list->word->word);
 	  if (fatal)
 	    throw_to_top_level ();
 	  else
@@ -362,13 +403,19 @@ get_exitstat (list)
      WORD_LIST *list;
 {
   int status;
-  long sval;
+  intmax_t sval;
   char *arg;
 
+  if (list && list->word && ISOPTION (list->word->word, '-'))
+    list = list->next;
+
+  if (list == 0)
+    return (last_command_exit_value);      
+
   arg = list->word->word;
   if (arg == 0 || legal_number (arg, &sval) == 0)
     {
-      builtin_error ("bad non-numeric arg `%s'", list->word->word);
+      sh_neednumarg (list->word->word ? list->word->word : "`'");
       return 255;
     }
   no_args (list->next);
@@ -415,28 +462,22 @@ get_working_directory (for_whom)
      char *for_whom;
 {
   char *directory;
+  size_t dsize;
 
   if (no_symbolic_links)
     {
-      if (the_current_working_directory)
-	free (the_current_working_directory);
-
+      FREE (the_current_working_directory);
       the_current_working_directory = (char *)NULL;
     }
 
   if (the_current_working_directory == 0)
     {
-      the_current_working_directory = (char *)xmalloc (PATH_MAX);
-      the_current_working_directory[0] = '\0';
-      directory = getcwd (the_current_working_directory, PATH_MAX);
-      if (directory == 0)
+      the_current_working_directory = getcwd (0, 0);
+      if (the_current_working_directory == 0)
 	{
-	  fprintf (stderr, "%s: could not get current directory: %s: %s\n",
+	  fprintf (stderr, _("%s: error retrieving current directory: %s: %s\n"),
 		   (for_whom && *for_whom) ? for_whom : get_name_for_error (),
-		   bash_getcwd_errstr, strerror (errno));
-
-	  free (the_current_working_directory);
-	  the_current_working_directory = (char *)NULL;
+		   _(bash_getcwd_errstr), strerror (errno));
 	  return (char *)NULL;
 	}
     }
@@ -460,13 +501,65 @@ set_working_directory (name)
 /* **************************************************************** */
 
 #if defined (JOB_CONTROL)
+int
+get_job_by_name (name, flags)
+     const char *name;
+     int flags;
+{
+  register int i, wl, cl, match, job;
+  register PROCESS *p;
+
+  job = NO_JOB;
+  wl = strlen (name);
+  for (i = job_slots - 1; i >= 0; i--)
+    {
+      if (jobs[i] == 0 || ((flags & JM_STOPPED) && JOBSTATE(i) != JSTOPPED))
+        continue;
+
+      p = jobs[i]->pipe;
+      do
+        {
+	  if (flags & JM_EXACT)
+	    {
+	      cl = strlen (p->command);
+	      match = STREQN (p->command, name, cl);
+	    }
+	  else if (flags & JM_SUBSTRING)
+	    match = strindex (p->command, name) != (char *)0;
+	  else
+	    match = STREQN (p->command, name, wl);
+
+	  if (match == 0)
+	    {
+	      p = p->next;
+	      continue;
+	    }
+	  else if (flags & JM_FIRSTMATCH)
+	    return i;		/* return first match */
+	  else if (job != NO_JOB)
+	    {
+	      if (this_shell_builtin)
+	        builtin_error (_("%s: ambiguous job spec"), name);
+	      else
+	        report_error (_("%s: ambiguous job spec"), name);
+	      return (DUP_JOB);
+	    }
+	  else
+	    job = i;
+        }
+      while (p != jobs[i]->pipe);
+    }
+
+  return (job);
+}
+
 /* Return the job spec found in LIST. */
 int
 get_job_spec (list)
      WORD_LIST *list;
 {
   register char *word;
-  int job, substring_search;
+  int job, jflags;
 
   if (list == 0)
     return (current_job);
@@ -474,7 +567,7 @@ get_job_spec (list)
   word = list->word->word;
 
   if (*word == '\0')
-    return (current_job);
+    return (NO_JOB);
 
   if (*word == '%')
     word++;
@@ -482,13 +575,14 @@ get_job_spec (list)
   if (DIGIT (*word) && all_digits (word))
     {
       job = atoi (word);
-      return (job >= job_slots ? NO_JOB : job - 1);
+      return (job > job_slots ? NO_JOB : job - 1);
     }
 
-  substring_search = 0;
+  jflags = 0;
   switch (*word)
     {
     case 0:
+	return NO_JOB;
     case '%':
     case '+':
       return (current_job);
@@ -497,47 +591,19 @@ get_job_spec (list)
       return (previous_job);
 
     case '?':			/* Substring search requested. */
-      substring_search++;
+      jflags |= JM_SUBSTRING;
       word++;
       /* FALLTHROUGH */
 
     default:
-      {
-	register int i, wl;
-
-	job = NO_JOB;
-	wl = strlen (word);
-	for (i = 0; i < job_slots; i++)
-	  {
-	    if (jobs[i])
-	      {
-		register PROCESS *p;
-		p = jobs[i]->pipe;
-		do
-		  {
-		    if ((substring_search && strindex (p->command, word)) ||
-			(STREQN (p->command, word, wl)))
-		      {
-			if (job != NO_JOB)
-			  {
-			    builtin_error ("ambigious job spec: %s", word);
-			    return (DUP_JOB);
-			  }
-			else
-			  job = i;
-		      }
-
-		    p = p->next;
-		  }
-		while (p != jobs[i]->pipe);
-	      }
-	  }
-	return (job);
-      }
+      return get_job_by_name (word, jflags);
     }
 }
 #endif /* JOB_CONTROL */
 
+/*
+ * NOTE:  `kill' calls this function with forcecols == 0
+ */
 int
 display_signal_list (list, forcecols)
      WORD_LIST *list;
@@ -545,8 +611,8 @@ display_signal_list (list, forcecols)
 {
   register int i, column;
   char *name;
-  int result;
-  long signum;
+  int result, signum, dflags;
+  intmax_t lsignum;
 
   result = EXECUTION_SUCCESS;
   if (!list)
@@ -558,7 +624,13 @@ display_signal_list (list, forcecols)
 	    continue;
 
 	  if (posixly_correct && !forcecols)
-	    printf ("%s%s", name, (i == NSIG - 1) ? "" : " ");
+	    {
+	      /* This is for the kill builtin.  POSIX.2 says the signal names
+		 are displayed without the `SIG' prefix. */
+	      if (STREQN (name, "SIG", 3))
+		name += 3;
+	      printf ("%s%s", name, (i == NSIG - 1) ? "" : " ");
+	    }
 	  else
 	    {
 	      printf ("%2d) %s", i, name);
@@ -581,20 +653,21 @@ display_signal_list (list, forcecols)
   /* List individual signal names or numbers. */
   while (list)
     {
-      if (legal_number (list->word->word, &signum))
+      if (legal_number (list->word->word, &lsignum))
 	{
 	  /* This is specified by Posix.2 so that exit statuses can be
 	     mapped into signal numbers. */
-	  if (signum > 128)
-	    signum -= 128;
-	  if (signum < 0 || signum >= NSIG)
+	  if (lsignum > 128)
+	    lsignum -= 128;
+	  if (lsignum < 0 || lsignum >= NSIG)
 	    {
-	      builtin_error ("bad signal number: %s", list->word->word);
+	      sh_invalidsig (list->word->word);
 	      result = EXECUTION_FAILURE;
 	      list = list->next;
 	      continue;
 	    }
 
+	  signum = lsignum;
 	  name = signal_name (signum);
 	  if (STREQN (name, "SIGJUNK", 7) || STREQN (name, "Unknown", 7))
 	    {
@@ -611,15 +684,18 @@ display_signal_list (list, forcecols)
 	}
       else
 	{
-	  signum = decode_signal (list->word->word);
+	  dflags = DSIG_NOCASE;
+	  if (posixly_correct == 0 || this_shell_builtin != kill_builtin)
+	    dflags |= DSIG_SIGPREFIX;
+	  signum = decode_signal (list->word->word, dflags);
 	  if (signum == NO_SIG)
 	    {
-	      builtin_error ("%s: not a signal specification", list->word->word);
+	      sh_invalidsig (list->word->word);
 	      result = EXECUTION_FAILURE;
 	      list = list->next;
 	      continue;
 	    }
-	  printf ("%ld\n", signum);
+	  printf ("%d\n", signum);
 	}
       list = list->next;
     }
