@@ -1,6 +1,6 @@
 /* common.h -- extern declarations for functions defined in common.c. */
 
-/* Copyright (C) 1993-2002 Free Software Foundation, Inc.
+/* Copyright (C) 1993-2004 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -30,6 +30,7 @@
 #define SEVAL_INTERACT	0x002
 #define SEVAL_NOHIST	0x004
 #define SEVAL_NOFREE	0x008
+#define SEVAL_RESETLINE	0x010
 
 /* Flags for describe_command, shared between type.def and command.def */
 #define CDESC_ALL		0x001	/* type -a */
@@ -74,6 +75,7 @@ extern void sh_badjob __P((char *));
 extern void sh_readonly __P((const char *));
 extern void sh_nojobs __P((char *));
 extern void sh_restricted __P((char *));
+extern void sh_notbuiltin __P((char *));
 
 extern char **make_builtin_argv __P((WORD_LIST *, int *));
 extern void remember_args __P((WORD_LIST *, int));
@@ -105,6 +107,9 @@ extern sh_builtin_func_t *builtin_address __P((char *));
 extern sh_builtin_func_t *find_special_builtin __P((char *));
 extern void initialize_shell_builtins __P((void));
 
+/* Functions from exit.def */
+extern void bash_logout __P((void));
+
 /* Functions from getopts.def */
 extern void getopts_reset __P((int));
 
@@ -150,7 +155,7 @@ extern void parse_and_execute_cleanup __P((void));
 
 /* Functions from evalfile.c */
 extern int maybe_execute_file __P((const char *, int));
-extern int source_file __P((const char *));
+extern int source_file __P((const char *, int));
 extern int fc_execute_file __P((const char *));
 
 #endif /* !__COMMON_H */
