@@ -27,10 +27,11 @@
 /* Instructions describing what kind of thing to do for a redirection. */
 enum r_instruction {
   r_output_direction, r_input_direction, r_inputa_direction,
-  r_appending_to, r_reading_until, r_duplicating_input,
-  r_duplicating_output, r_deblank_reading_until, r_close_this,
-  r_err_and_out, r_input_output, r_output_force,
-  r_duplicating_input_word, r_duplicating_output_word
+  r_appending_to, r_reading_until, r_reading_string,
+  r_duplicating_input, r_duplicating_output, r_deblank_reading_until,
+  r_close_this, r_err_and_out, r_input_output, r_output_force,
+  r_duplicating_input_word, r_duplicating_output_word,
+  r_move_input, r_move_output, r_move_input_word, r_move_output_word
 };
 
 /* Redirection errors. */
@@ -55,19 +56,28 @@ enum r_instruction {
 	ri == r_appending_to || \
 	ri == r_output_force)
 
+/* redirection needs translation */
+#define TRANSLATE_REDIRECT(ri) \
+  (ri == r_duplicating_input_word || ri == r_duplicating_output_word || \
+   ri == r_move_input_word || ri == r_move_output_word)
+
 /* Command Types: */
 enum command_type { cm_for, cm_case, cm_while, cm_if, cm_simple, cm_select,
 		    cm_connection, cm_function_def, cm_until, cm_group,
 		    cm_arith, cm_cond, cm_arith_for, cm_subshell };
 
 /* Possible values for the `flags' field of a WORD_DESC. */
-#define W_HASDOLLAR	0x01	/* Dollar sign present. */
-#define W_QUOTED	0x02	/* Some form of quote character is present. */
-#define W_ASSIGNMENT	0x04	/* This word is a variable assignment. */
-#define W_GLOBEXP	0x08	/* This word is the result of a glob expansion. */
-#define W_NOSPLIT	0x10	/* Do not perform word splitting on this word. */
-#define W_NOGLOB	0x20	/* Do not perform globbing on this word. */
-#define W_NOSPLIT2	0x40	/* Don't split word except for $@ expansion. */
+#define W_HASDOLLAR	0x0001	/* Dollar sign present. */
+#define W_QUOTED	0x0002	/* Some form of quote character is present. */
+#define W_ASSIGNMENT	0x0004	/* This word is a variable assignment. */
+#define W_GLOBEXP	0x0008	/* This word is the result of a glob expansion. */
+#define W_NOSPLIT	0x0010	/* Do not perform word splitting on this word. */
+#define W_NOGLOB	0x0020	/* Do not perform globbing on this word. */
+#define W_NOSPLIT2	0x0040	/* Don't split word except for $@ expansion. */
+#define W_TILDEEXP	0x0080	/* Tilde expand this assignment word */
+#define W_DOLLARAT	0x0100	/* $@ and its special handling */
+#define W_DOLLARSTAR	0x0200	/* $* and its special handling */
+#define W_NOCOMSUB	0x0400	/* Don't perform command substitution on this word */
 
 /* Possible values for subshell_environment */
 #define SUBSHELL_ASYNC	0x01	/* subshell caused by `command &' */
@@ -95,14 +105,18 @@ typedef struct word_list {
 /*								    */
 /* **************************************************************** */
 
-/* What a redirection descriptor looks like.  If FLAGS is IS_DESCRIPTOR,
-   then we use REDIRECTEE.DEST, else we use the file specified. */
+/* What a redirection descriptor looks like.  If the redirection instruction
+   is ri_duplicating_input or ri_duplicating_output, use DEST, otherwise
+   use the file in FILENAME.  Out-of-range descriptors are identified by a
+   negative DEST. */
 
 typedef union {
-  long dest;			/* Place to redirect REDIRECTOR to, or ... */
+  int dest;			/* Place to redirect REDIRECTOR to, or ... */
   WORD_DESC *filename;		/* filename to redirect to. */
 } REDIRECTEE;
 
+/* Structure describing a redirection.  If REDIRECTOR is negative, the parser
+   (or translator in redir.c) encountered an out-of-range file descriptor. */
 typedef struct redirect {
   struct redirect *next;	/* Next element, or NULL. */
   int redirector;		/* Descriptor to be redirected. */
@@ -131,6 +145,7 @@ typedef struct element {
 #define CMD_TIME_POSIX	   0x100 /* time -p; use POSIX.2 time output spec. */
 #define CMD_AMPERSAND	   0x200 /* command & */
 #define CMD_STDIN_REDIR	   0x400 /* async command needs implicit </dev/null */
+#define CMD_COMMAND_BUILTIN 0x0800 /* command executed by `command' builtin */
 
 /* What a command looks like. */
 typedef struct command {
@@ -183,6 +198,7 @@ typedef struct pattern_list {
 /* The CASE command. */
 typedef struct case_com {
   int flags;			/* See description of CMD flags. */
+  int line;			/* line number the `case' keyword appears on */
   WORD_DESC *word;		/* The thing to test. */
   PATTERN_LIST *clauses;	/* The clauses to test against, or NULL. */
 } CASE_COM;
@@ -190,6 +206,7 @@ typedef struct case_com {
 /* FOR command. */
 typedef struct for_com {
   int flags;		/* See description of CMD flags. */
+  int line;		/* line number the `for' keyword appears on */
   WORD_DESC *name;	/* The variable name to get mapped over. */
   WORD_LIST *map_list;	/* The things to map over.  This is never NULL. */
   COMMAND *action;	/* The action to execute.
@@ -212,6 +229,7 @@ typedef struct arith_for_com {
 /* KSH SELECT command. */
 typedef struct select_com {
   int flags;		/* See description of CMD flags. */
+  int line;		/* line number the `select' keyword appears on */
   WORD_DESC *name;	/* The variable name to get mapped over. */
   WORD_LIST *map_list;	/* The things to map over.  This is never NULL. */
   COMMAND *action;	/* The action to execute.
@@ -241,8 +259,8 @@ typedef struct while_com {
    time being. */
 typedef struct arith_com {
   int flags;
-  WORD_LIST *exp;
   int line;
+  WORD_LIST *exp;
 } ARITH_COM;
 #endif /* DPAREN_ARITHMETIC */
 
@@ -266,18 +284,19 @@ typedef struct cond_com {
 /* The "simple" command.  Just a collection of words and redirects. */
 typedef struct simple_com {
   int flags;			/* See description of CMD flags. */
+  int line;			/* line number the command starts on */
   WORD_LIST *words;		/* The program name, the arguments,
 				   variable assignments, etc. */
   REDIRECT *redirects;		/* Redirections to perform. */
-  int line;			/* line number the command starts on */
 } SIMPLE_COM;
 
 /* The "function definition" command. */
 typedef struct function_def {
   int flags;			/* See description of CMD flags. */
+  int line;			/* Line number the function def starts on. */
   WORD_DESC *name;		/* The name of the function. */
   COMMAND *command;		/* The parsed execution tree. */
-  int line;			/* Line number the function def starts on. */
+  char *source_file;		/* file in which function was defined, if any */
 } FUNCTION_DEF;
 
 /* A command that is `grouped' allows pipes and redirections to affect all
@@ -304,6 +323,9 @@ extern COMMAND *global_command;
 
 /* Forward declarations of functions declared in copy_cmd.c. */
 
+extern FUNCTION_DEF *copy_function_def_contents __P((FUNCTION_DEF *, FUNCTION_DEF *));
+extern FUNCTION_DEF *copy_function_def __P((FUNCTION_DEF *));
+
 extern WORD_DESC *copy_word __P((WORD_DESC *));
 extern WORD_LIST *copy_word_list __P((WORD_LIST *));
 extern REDIRECT *copy_redirect __P((REDIRECT *));
