@@ -16,7 +16,7 @@
 
    You should have received a copy of the GNU General Public License along
    with Bash; see the file COPYING.  If not, write to the Free Software
-   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+   Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
 #if !defined (_DISPOSE_CMD_H_)
 #define _DISPOSE_CMD_H_
@@ -29,4 +29,11 @@ extern void dispose_words __P((WORD_LIST *));
 extern void dispose_word_array __P((char **));
 extern void dispose_redirects __P((REDIRECT *));
 
+#if defined (COND_COMMAND)
+extern void dispose_cond_node __P((COND_COM *));
+#endif
+
+extern void dispose_function_def_contents __P((FUNCTION_DEF *));
+extern void dispose_function_def __P((FUNCTION_DEF *));
+
 #endif /* !_DISPOSE_CMD_H_ */
