@@ -2,6 +2,10 @@
  * finfo - print file info
  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <sys/types.h>
 #include "posixstat.h"
 #include <stdio.h>
@@ -127,10 +131,10 @@ char	*f;
 {
 	static struct stat st;
 	int	fd, r;
-	long	lfd;
+	intmax_t lfd;
 
 	if (strncmp(f, "/dev/fd/", 8) == 0) {
-		if (legal_number(f + 8, &lfd) == 0) {
+		if ((legal_number(f + 8, &lfd) == 0) || (int)lfd != lfd) {
 			builtin_error("%s: invalid fd", f + 8);
 			return ((struct stat *)0);
 		}
@@ -243,10 +247,17 @@ struct stat *st;
 	struct passwd	*pw;
 	struct group	*gr;
 	char	*owner;
+	int	ma, mi, d;
+
+	ma = major (st->st_rdev);
+	mi = minor (st->st_rdev);
+#if defined (makedev)
+	d = makedev (ma, mi);
+#else
+	d = st->st_rdev & 0xFF;
+#endif
+	printf("Device (major/minor): %d (%d/%d)\n", d, ma, mi);
 
-	printf("Device (major/minor): %d (%d/%d)\n", (int) (st->st_dev & 0xFF),
-						     (int) major (st->st_dev),
-						     (int) minor (st->st_dev));
 	printf("Inode: %d\n", (int) st->st_ino);
 	printf("Mode: (%o) ", (int) st->st_mode);
 	printmode((int) st->st_mode);
@@ -348,6 +359,7 @@ int	flags;
 }
 
 #ifndef NOBUILTIN
+int
 finfo_builtin(list)
      WORD_LIST *list;
 {
