@@ -1,6 +1,6 @@
 /* expr.c -- arithmetic expression evaluation. */
 
-/* Copyright (C) 1990, 1991 Free Software Foundation, Inc.
+/* Copyright (C) 1990-2004 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -19,7 +19,7 @@
    Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 
 /*
- All arithmetic is done as long integers with no checking for overflow
+ All arithmetic is done as intmax_t integers with no checking for overflow
  (though division by 0 is caught and flagged as an error).
 
  The following operators are handled, grouped into a set of levels in
@@ -42,6 +42,7 @@
 	"||"
 	"expr ? expr : expr"
 	"=", "*=", "/=", "%=", "+=", "-=", "<<=", ">>=", "&=", "^=", "|="
+	,			[comma]
 
  (Note that most of these operators have special meaning to bash, and an
  entire expression should be quoted, e.g. "a=$a+1" or "a=a+1" to ensure
@@ -78,6 +79,7 @@
 #endif
 
 #include "chartypes.h"
+#include "bashintl.h"
 
 #include "shell.h"
 
@@ -142,41 +144,45 @@ static int	curtok;		/* the current token */
 static int	lasttok;	/* the previous token */
 static int	assigntok;	/* the OP in OP= */
 static char	*tokstr;	/* current token string */
-static long	tokval;		/* current token value */
+static intmax_t	tokval;		/* current token value */
 static int	noeval;		/* set to 1 if no assignment to be done */
 static procenv_t evalbuf;
 
 static void	readtok __P((void));	/* lexical analyzer */
-static long	strlong __P((char *));
+
+static intmax_t	expr_streval __P((char *, int));
+static intmax_t	strlong __P((char *));
 static void	evalerror __P((char *));
 
 static void	pushexp __P((void));
 static void	popexp __P((void));
+static void	expr_unwind __P((void));
+static void	expr_bind_variable __P((char *, char *));
 
-static long 	subexpr __P((char *));
+static intmax_t subexpr __P((char *));
 
-static long	expcomma __P((void));
-static long 	expassign __P((void));
-static long	expcond __P((void));
-static long 	explor __P((void));
-static long 	expland __P((void));
-static long	expbor __P((void));
-static long	expbxor __P((void));
-static long	expband __P((void));
-static long 	exp5 __P((void));
-static long 	exp4 __P((void));
-static long 	expshift __P((void));
-static long 	exp3 __P((void));
-static long 	exp2 __P((void));
-static long	exppower __P((void));
-static long 	exp1 __P((void));
-static long 	exp0 __P((void));
+static intmax_t	expcomma __P((void));
+static intmax_t expassign __P((void));
+static intmax_t	expcond __P((void));
+static intmax_t explor __P((void));
+static intmax_t expland __P((void));
+static intmax_t	expbor __P((void));
+static intmax_t	expbxor __P((void));
+static intmax_t	expband __P((void));
+static intmax_t exp5 __P((void));
+static intmax_t exp4 __P((void));
+static intmax_t expshift __P((void));
+static intmax_t exp3 __P((void));
+static intmax_t exp2 __P((void));
+static intmax_t	exppower __P((void));
+static intmax_t exp1 __P((void));
+static intmax_t exp0 __P((void));
 
 /* A structure defining a single expression context. */
 typedef struct {
   int curtok, lasttok;
   char *expression, *tp, *lasttp;
-  long tokval;
+  intmax_t tokval;
   char *tokstr;
   int noeval;
 } EXPR_CONTEXT;
@@ -185,7 +191,7 @@ typedef struct {
 /* Not used yet. */
 typedef struct {
   char *tokstr;
-  long tokval;
+  intmax_t tokval;
 } LVALUE;
 #endif
 
@@ -195,6 +201,11 @@ static int expr_depth;		   /* Location in the stack. */
 static int expr_stack_size;	   /* Number of slots already allocated. */
 
 extern char *this_command_name;
+extern int unbound_vars_is_error;
+
+#if defined (ARRAY_VARS)
+extern char *bash_badsub_errmsg;
+#endif
 
 #define SAVETOK(X) \
   do { \
@@ -226,7 +237,7 @@ pushexp ()
   EXPR_CONTEXT *context;
 
   if (expr_depth >= MAX_EXPR_RECURSION_LEVEL)
-    evalerror ("expression recursion level exceeded");
+    evalerror (_("expression recursion level exceeded"));
 
   if (expr_depth >= expr_stack_size)
     {
@@ -250,7 +261,7 @@ popexp ()
   EXPR_CONTEXT *context;
 
   if (expr_depth == 0)
-    evalerror ("recursion stack underflow");
+    evalerror (_("recursion stack underflow"));
 
   context = expr_stack[--expr_depth];
 
@@ -260,6 +271,30 @@ popexp ()
   free (context);
 }
 
+static void
+expr_unwind ()
+{
+  while (--expr_depth > 0)
+    {
+      if (expr_stack[expr_depth]->tokstr)
+	free (expr_stack[expr_depth]->tokstr);
+
+      if (expr_stack[expr_depth]->expression)
+	free (expr_stack[expr_depth]->expression);
+
+      free (expr_stack[expr_depth]);
+    }
+  free (expr_stack[expr_depth]);	/* free the allocated EXPR_CONTEXT */
+}
+
+static void
+expr_bind_variable (lhs, rhs)
+     char *lhs, *rhs;
+{
+  (void)bind_int_variable (lhs, rhs);
+  stupidly_hack_special_variables (lhs);
+}
+
 /* Evaluate EXPR, and return the arithmetic result.  If VALIDP is
    non-null, a zero is stored into the location to which it points
    if the expression is invalid, non-zero otherwise.  If a non-zero
@@ -273,41 +308,28 @@ popexp ()
    were assigned at program startup or by the compiler.  Therefore, it is
    safe to let the loop terminate when expr_depth == 0, without freeing up
    any of the expr_depth[0] stuff. */
-long
+intmax_t
 evalexp (expr, validp)
      char *expr;
      int *validp;
 {
-  long val;
-#if 0
-  procenv_t old_evalbuf;
-#endif
+  intmax_t val;
+  int c;
+  procenv_t oevalbuf;
 
   val = 0;
 
-#if 0
-  /* Save the value of evalbuf to protect it around possible recursive
-     calls to evalexp (). */
-  COPY_PROCENV (evalbuf, old_evalbuf);
-#endif
+  FASTCOPY (evalbuf, oevalbuf, sizeof (evalbuf));
 
-  if (setjmp (evalbuf))
+  c = setjmp (evalbuf);
+
+  if (c)
     {
       FREE (tokstr);
       FREE (expression);
       tokstr = expression = (char *)NULL;
 
-      while (--expr_depth > 0)
-	{
-	  if (expr_stack[expr_depth]->tokstr)
-	    free (expr_stack[expr_depth]->tokstr);
-
-	  if (expr_stack[expr_depth]->expression)
-	    free (expr_stack[expr_depth]->expression);
-
-	  free (expr_stack[expr_depth]);
-	}
-      free (expr_stack[expr_depth]);	/* free the allocated EXPR_CONTEXT */
+      expr_unwind ();
 
       if (validp)
 	*validp = 0;
@@ -316,23 +338,19 @@ evalexp (expr, validp)
 
   val = subexpr (expr);
 
-#if 0
-  /* Restore the value of evalbuf so that any subsequent longjmp calls
-     will have a valid location to jump to. */
-  COPY_PROCENV (old_evalbuf, evalbuf);
-#endif
-
   if (validp)
     *validp = 1;
 
+  FASTCOPY (oevalbuf, evalbuf, sizeof (evalbuf));
+
   return (val);
 }
 
-static long
+static intmax_t
 subexpr (expr)
      char *expr;
 {
-  long val;
+  intmax_t val;
   char *p;
 
   for (p = expr; p && *p && cr_whitespace (*p); p++)
@@ -354,7 +372,7 @@ subexpr (expr)
   val = EXP_HIGHEST ();
 
   if (curtok != 0)
-    evalerror ("syntax error in expression");
+    evalerror (_("syntax error in expression"));
 
   FREE (tokstr);
   FREE (expression);
@@ -364,10 +382,10 @@ subexpr (expr)
   return val;
 }
 
-static long
+static intmax_t
 expcomma ()
 {
-  register long value;
+  register intmax_t value;
 
   value = expassign ();
   while (curtok == COMMA)
@@ -379,22 +397,22 @@ expcomma ()
   return value;
 }
   
-static long
+static intmax_t
 expassign ()
 {
-  register long	value;
+  register intmax_t value;
   char *lhs, *rhs;
 
   value = expcond ();
   if (curtok == EQ || curtok == OP_ASSIGN)
     {
       int special, op;
-      long lvalue;
+      intmax_t lvalue;
 
       special = curtok == OP_ASSIGN;
 
       if (lasttok != STR)
-	evalerror ("attempted assignment to non-variable");
+	evalerror (_("attempted assignment to non-variable"));
 
       if (special)
 	{
@@ -414,9 +432,13 @@ expassign ()
 	      lvalue *= value;
 	      break;
 	    case DIV:
+	      if (value == 0)
+		evalerror (_("division by 0"));
 	      lvalue /= value;
 	      break;
 	    case MOD:
+	      if (value == 0)
+		evalerror (_("division by 0"));
 	      lvalue %= value;
 	      break;
 	    case PLUS:
@@ -442,7 +464,7 @@ expassign ()
 	      break;
 	    default:
 	      free (lhs);
-	      evalerror ("bug: bad expassign token");
+	      evalerror (_("bug: bad expassign token"));
 	      break;
 	    }
 	  value = lvalue;
@@ -450,7 +472,7 @@ expassign ()
 
       rhs = itos (value);
       if (noeval == 0)
-	(void)bind_int_variable (lhs, rhs);
+	expr_bind_variable (lhs, rhs);
       free (rhs);
       free (lhs);
       FREE (tokstr);
@@ -460,10 +482,10 @@ expassign ()
 }
 
 /* Conditional expression (expr?expr:expr) */
-static long
+static intmax_t
 expcond ()
 {
-  long cval, val1, val2, rval;
+  intmax_t cval, val1, val2, rval;
   int set_noeval;
 
   set_noeval = 0;
@@ -472,7 +494,7 @@ expcond ()
     {
       readtok ();
       if (curtok == 0 || curtok == COL)
-	evalerror ("expression expected");
+	evalerror (_("expression expected"));
       if (cval == 0)
 	{
 	  set_noeval = 1;
@@ -484,10 +506,10 @@ expcond ()
       if (set_noeval)
 	noeval--;
       if (curtok != COL)
-	evalerror ("`:' expected for conditional expression");
+	evalerror (_("`:' expected for conditional expression"));
       readtok ();
       if (curtok == 0)
-	evalerror ("expression expected");
+	evalerror (_("expression expected"));
       set_noeval = 0;
       if (cval)
  	{
@@ -504,10 +526,10 @@ expcond ()
 }
 
 /* Logical OR. */
-static long
+static intmax_t
 explor ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
   int set_noeval;
 
   val1 = expland ();
@@ -532,10 +554,10 @@ explor ()
 }
 
 /* Logical AND. */
-static long
+static intmax_t
 expland ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
   int set_noeval;
 
   val1 = expbor ();
@@ -560,10 +582,10 @@ expland ()
 }
 
 /* Bitwise OR. */
-static long
+static intmax_t
 expbor ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = expbxor ();
 
@@ -578,10 +600,10 @@ expbor ()
 }
 
 /* Bitwise XOR. */
-static long
+static intmax_t
 expbxor ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = expband ();
 
@@ -596,10 +618,10 @@ expbxor ()
 }
 
 /* Bitwise AND. */
-static long
+static intmax_t
 expband ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = exp5 ();
 
@@ -613,10 +635,10 @@ expband ()
   return (val1);
 }
 
-static long
+static intmax_t
 exp5 ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = exp4 ();
 
@@ -634,10 +656,10 @@ exp5 ()
   return (val1);
 }
 
-static long
+static intmax_t
 exp4 ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = expshift ();
   while ((curtok == LEQ) ||
@@ -663,10 +685,10 @@ exp4 ()
 }
 
 /* Left and right shifts. */
-static long
+static intmax_t
 expshift ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = exp3 ();
 
@@ -686,10 +708,10 @@ expshift ()
   return (val1);
 }
 
-static long
+static intmax_t
 exp3 ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = exp2 ();
 
@@ -708,10 +730,10 @@ exp3 ()
   return (val1);
 }
 
-static long
+static intmax_t
 exp2 ()
 {
-  register long val1, val2;
+  register intmax_t val1, val2;
 
   val1 = exppower ();
 
@@ -726,7 +748,7 @@ exp2 ()
       val2 = exppower ();
 
       if (((op == DIV) || (op == MOD)) && (val2 == 0))
-	evalerror ("division by 0");
+	evalerror (_("division by 0"));
 
       if (op == MUL)
 	val1 *= val2;
@@ -738,20 +760,20 @@ exp2 ()
   return (val1);
 }
 
-static long
+static intmax_t
 exppower ()
 {
-  register long val1, val2, c;
+  register intmax_t val1, val2, c;
 
   val1 = exp1 ();
-  if (curtok == POWER)
+  while (curtok == POWER)
     {
       readtok ();
       val2 = exp1 ();
       if (val2 == 0)
 	return (1);
       if (val2 < 0)
-	evalerror ("exponent less than 0");
+	evalerror (_("exponent less than 0"));
       for (c = 1; val2--; c *= val1)
 	;
       val1 = c;
@@ -759,10 +781,10 @@ exppower ()
   return (val1);
 }
 
-static long
+static intmax_t
 exp1 ()
 {
-  register long val;
+  register intmax_t val;
 
   if (curtok == NOT)
     {
@@ -780,12 +802,13 @@ exp1 ()
   return (val);
 }
 
-static long
+static intmax_t
 exp0 ()
 {
-  register long val = 0, v2;
+  register intmax_t val = 0, v2;
   char *vincdec;
   int stok;
+  EXPR_CONTEXT ec;
 
   /* XXX - might need additional logic here to decide whether or not
 	   pre-increment or pre-decrement is legal at this point. */
@@ -795,12 +818,12 @@ exp0 ()
       readtok ();
       if (curtok != STR)
 	/* readtok() catches this */
-	evalerror ("identifier expected after pre-increment or pre-decrement");
+	evalerror (_("identifier expected after pre-increment or pre-decrement"));
 
       v2 = tokval + ((stok == PREINC) ? 1 : -1);
       vincdec = itos (v2);
       if (noeval == 0)
-	(void)bind_int_variable (tokstr, vincdec);
+	expr_bind_variable (tokstr, vincdec);
       free (vincdec);
       val = v2;
 
@@ -822,8 +845,8 @@ exp0 ()
       readtok ();
       val = EXP_HIGHEST ();
 
-      if (curtok != RPAR)
-	evalerror ("missing `)'");
+      if (curtok != RPAR) /* ( */
+	evalerror (_("missing `)'"));
 
       /* Skip over closing paren. */
       readtok ();
@@ -831,27 +854,101 @@ exp0 ()
   else if ((curtok == NUM) || (curtok == STR))
     {
       val = tokval;
-      if (curtok == STR && (*tp == '+' || *tp == '-') && tp[1] == *tp &&
-		(tp[2] == '\0' || (ISALNUM ((unsigned char)tp[2]) == 0)))
+      if (curtok == STR)
 	{
+	  SAVETOK (&ec);
+	  tokstr = (char *)NULL;	/* keep it from being freed */
+          noeval = 1;
+          readtok ();
+          stok = curtok;
+
 	  /* post-increment or post-decrement */
-	  v2 = val + ((*tp == '+') ? 1 : -1);
-	  vincdec = itos (v2);
-	  if (noeval == 0)
-	    (void)bind_int_variable (tokstr, vincdec);
-	  free (vincdec);
-	  tp += 2;
-	  curtok = NUM;	/* make sure x++=7 is flagged as an error */
+ 	  if (stok == POSTINC || stok == POSTDEC)
+ 	    {
+ 	      /* restore certain portions of EC */
+ 	      tokstr = ec.tokstr;
+ 	      noeval = ec.noeval;
+ 	      lasttok = STR;	/* ec.curtok */
+
+	      v2 = val + ((stok == POSTINC) ? 1 : -1);
+	      vincdec = itos (v2);
+	      if (noeval == 0)
+		expr_bind_variable (tokstr, vincdec);
+	      free (vincdec);
+	      curtok = NUM;	/* make sure x++=7 is flagged as an error */
+ 	    }
+ 	  else
+ 	    {
+	      if (stok == STR)	/* free new tokstr before old one is restored */
+		FREE (tokstr);
+	      RESTORETOK (&ec);
+ 	    }
+
 	}
 	  
       readtok ();
     }
   else
-    evalerror ("syntax error: operand expected");
+    evalerror (_("syntax error: operand expected"));
 
   return (val);
 }
 
+static intmax_t
+expr_streval (tok, e)
+     char *tok;
+     int e;
+{
+  SHELL_VAR *v;
+  char *value;
+  intmax_t tval;
+
+  /* [[[[[ */
+#if defined (ARRAY_VARS)
+  v = (e == ']') ? array_variable_part (tok, (char **)0, (int *)0) : find_variable (tok);
+#else
+  v = find_variable (tok);
+#endif
+
+  if ((v == 0 || invisible_p (v)) && unbound_vars_is_error)
+    {
+#if defined (ARRAY_VARS)
+      value = (e == ']') ? array_variable_name (tok, (char **)0, (int *)0) : tok;
+#else
+      value = tok;
+#endif
+
+      err_unboundvar (value);
+
+#if defined (ARRAY_VARS)
+      if (e == ']')
+	FREE (value);	/* array_variable_name returns new memory */
+#endif
+
+      if (interactive_shell)
+	{
+	  expr_unwind ();
+	  jump_to_top_level (DISCARD);
+	}
+      else
+	jump_to_top_level (FORCE_EOF);
+    }
+
+#if defined (ARRAY_VARS)
+  /* Second argument of 0 to get_array_value means that we don't allow
+     references like array[@].  In this case, get_array_value is just
+     like get_variable_value in that it does not return newly-allocated
+     memory or quote the results. */
+  value = (e == ']') ? get_array_value (tok, 0, (int *)NULL) : get_variable_value (v);
+#else
+  value = get_variable_value (v);
+#endif
+
+  tval = (value && *value) ? subexpr (value) : 0;
+
+  return (tval);
+}
+
 /* Lexical analyzer/token reader for the expression evaluator.  Reads the
    next token and puts its value into curtok, while advancing past it.
    Updates value of tp.  May also set tokval (for number) or tokstr (for
@@ -859,7 +956,7 @@ exp0 ()
 static void
 readtok ()
 {
-  register char *cp;
+  register char *cp, *xp;
   register unsigned char c, c1;
   register int e;
 
@@ -885,7 +982,7 @@ readtok ()
   if (legal_variable_starter (c))
     {
       /* variable names not preceded with a dollar sign are shell variables. */
-      char *value, *savecp;
+      char *savecp;
       EXPR_CONTEXT ec;
       int peektok;
 
@@ -905,7 +1002,7 @@ readtok ()
 	      e = ']';
 	    }
 	  else
-	    evalerror ("bad array subscript");
+	    evalerror (bash_badsub_errmsg);
 	}
 #endif /* ARRAY_VARS */
 
@@ -918,6 +1015,7 @@ readtok ()
       tokstr = (char *)NULL;	/* keep it from being freed */
       tp = savecp = cp;
       noeval = 1;
+      curtok = STR;
       readtok ();
       peektok = curtok;
       if (peektok == STR)	/* free new tokstr before old one is restored */
@@ -928,20 +1026,7 @@ readtok ()
       /* The tests for PREINC and PREDEC aren't strictly correct, but they
 	 preserve old behavior if a construct like --x=9 is given. */
       if (lasttok == PREINC || lasttok == PREDEC || peektok != EQ)
-	{
-#if defined (ARRAY_VARS)
-	  value = (e == ']') ? get_array_value (tokstr, 0) : get_string_value (tokstr);
-#else
-	  value = get_string_value (tokstr);
-#endif
-
-	  tokval = (value && *value) ? subexpr (value) : 0;
-
-#if defined (ARRAY_VARS)
-	  if (e == ']')
-	    FREE (value);	/* get_array_value returns newly-allocated memory */
-#endif
-	}
+	tokval = expr_streval (tokstr, e);
       else
 	tokval = 0;
 
@@ -1000,10 +1085,20 @@ readtok ()
 	c = LOR;
       else if ((c == '*') && (c1 == '*'))
 	c = POWER;
-      else if ((c == '-') && (c1 == '-') && legal_variable_starter ((unsigned char)*cp))
-	c = PREDEC;
-      else if ((c == '+') && (c1 == '+') && legal_variable_starter ((unsigned char)*cp))
-	c = PREINC;
+      else if ((c == '-' || c == '+') && c1 == c && curtok == STR)
+	c = (c == '-') ? POSTDEC : POSTINC;
+      else if ((c == '-' || c == '+') && c1 == c)
+	{
+	  /* Quickly scan forward to see if this is followed by optional
+	     whitespace and an identifier. */
+	  xp = cp;
+	  while (xp && *xp && cr_whitespace (*xp))
+	    xp++;
+	  if (legal_variable_starter ((unsigned char)*xp))
+	    c = (c == '-') ? PREDEC : PREINC;
+	  else
+	    cp--;	/* not preinc or predec, so unget the character */
+	}
       else if (c1 == EQ && member (c, "*/%+-&^|"))
 	{
 	  assigntok = c;	/* a OP= b */
@@ -1032,7 +1127,7 @@ evalerror (msg)
   longjmp (evalbuf, 1);
 }
 
-/* Convert a string to a long integer, with an arbitrary base.
+/* Convert a string to an intmax_t integer, with an arbitrary base.
    0nnn -> base 8
    0[Xx]nn -> base 16
    Anything else: [base#]number (this is implemented to match ksh93)
@@ -1043,14 +1138,14 @@ evalerror (msg)
    from [0-9][a-z][A-Z]_@ (a = 10, z = 35, A = 36, Z = 61, _ = 62, @ = 63 --
    you get the picture). */
 
-static long
+static intmax_t
 strlong (num)
      char *num;
 {
   register char *s;
   register unsigned char c;
   int base, foundbase;
-  long val;
+  intmax_t val;
 
   s = num;
 
@@ -1080,11 +1175,11 @@ strlong (num)
       if (c == '#')
 	{
 	  if (foundbase)
-	    evalerror ("bad number");
+	    evalerror (_("invalid number"));
 
 	  /* Illegal base specifications raise an evaluation error. */
 	  if (val < 2 || val > 64)
-	    evalerror ("illegal arithmetic base");
+	    evalerror (_("invalid arithmetic base"));
 
 	  base = val;
 	  val = 0;
@@ -1104,7 +1199,7 @@ strlong (num)
 	    c = 63;
 
 	  if (c >= base)
-	    evalerror ("value too great for base");
+	    evalerror (_("value too great for base"));
 
 	  val = (val * base) + c;
 	}
@@ -1142,7 +1237,7 @@ main (argc, argv)
      char **argv;
 {
   register int i;
-  long v;
+  intmax_t v;
   int expok;
 
   if (setjmp (top_level))
@@ -1171,7 +1266,7 @@ builtin_error (format, arg1, arg2, arg3, arg4, arg5)
 
 char *
 itos (n)
-     long n;
+     intmax_t n;
 {
   return ("42");
 }
