@@ -52,22 +52,22 @@
 #  endif /* HAVE_SYS_PTEM_H && TIOCGWINSZ && SIGWINCH */
 #endif /* !STRUCT_WINSIZE_IN_SYS_IOCTL */
 
+#include "bashintl.h"
+
 #include "shell.h"
 #include "jobs.h"
 
 #include "builtins/builtext.h"	/* for wait_builtin */
 
-#if !defined (CHILD_MAX)
-#  define CHILD_MAX 32
-#endif
+#define DEFAULT_CHILD_MAX 32
 
 #if defined (_POSIX_VERSION) || !defined (HAVE_KILLPG)
 #  define killpg(pg, sig)		kill(-(pg),(sig))
 #endif /* USG || _POSIX_VERSION */
 
-#if !defined (HAVE_SIGINTERRUPT)
+#if !defined (HAVE_SIGINTERRUPT) && !defined (HAVE_POSIX_SIGNALS)
 #  define siginterrupt(sig, code)
-#endif /* !HAVE_SIGINTERRUPT */
+#endif /* !HAVE_SIGINTERRUPT && !HAVE_POSIX_SIGNALS */
 
 #if defined (HAVE_WAITPID)
 #  define WAITPID(pid, statusp, options) waitpid (pid, statusp, options)
@@ -88,13 +88,14 @@ extern void rl_set_screen_size __P((int, int));
 
 extern int interactive, interactive_shell, login_shell;
 extern int subshell_environment;
-extern int last_command_exit_value;
+extern int last_command_exit_value, last_command_exit_signal;
 extern int interrupt_immediately;
 extern sh_builtin_func_t *this_shell_builtin;
 #if defined (HAVE_POSIX_SIGNALS)
 extern sigset_t top_level_mask;
 #endif
 extern procenv_t wait_intr_buf;
+extern int wait_signal_received;
 
 pid_t last_made_pid = NO_PID;
 pid_t last_asynchronous_pid = NO_PID;
@@ -121,6 +122,7 @@ struct proc_status {
 #define PROC_RUNNING	0x01
 #define PROC_NOTIFIED	0x02
 #define PROC_ASYNC	0x04
+#define PROC_SIGNALED	0x10
 
 /* Return values from find_status_by_pid */
 #define PROC_BAD	 -1
@@ -130,25 +132,35 @@ static struct proc_status *pid_list = (struct proc_status *)NULL;
 static int pid_list_size;
 static int wait_sigint_received;
 
+static long child_max = -1L;
+
 static void alloc_pid_list __P((void));
 static int find_proc_slot __P((void));
 static int find_index_by_pid __P((pid_t));
 static int find_status_by_pid __P((pid_t));
 static int process_exit_status __P((WAIT));
+static int find_termsig_by_pid __P((pid_t));
+static int get_termsig __P((WAIT));
 static void set_pid_status __P((pid_t, WAIT));
 static void set_pid_flags __P((pid_t, int));
 static void unset_pid_flags __P((pid_t, int));
+static int get_pid_flags __P((pid_t));
 static void add_pid __P((pid_t, int));
 static void mark_dead_jobs_as_notified __P((int));
 
 static void get_new_window_size __P((int));
 static sighandler sigwinch_sighandler __P((int));
-static sighandler wait_signal_handler __P((int));
+static sighandler wait_sigint_handler __P((int));
+static char *j_strsignal __P((int));
 
 #if defined (HAVE_WAITPID)
 static void reap_zombie_children __P((void));
 #endif
 
+#if !defined (HAVE_SIGINTERRUPT) && defined (HAVE_POSIX_SIGNALS)
+static int siginterrupt __P((int, int));
+#endif
+
 static void restore_sigint_handler __P((void));
 
 /* Allocate new, or grow existing PID_LIST. */
@@ -224,6 +236,35 @@ process_exit_status (status)
     return (WEXITSTATUS (status));
 }
 
+/* Return the status of PID as looked up in the PID_LIST array.  A
+   return value of PROC_BAD indicates that PID wasn't found. */
+static int
+find_termsig_by_pid (pid)
+     pid_t pid;
+{
+  int i;
+
+  i = find_index_by_pid (pid);
+  if (i == NO_PID)
+    return (0);
+  if (pid_list[i].flags & PROC_RUNNING)
+    return (0);
+  return (get_termsig (pid_list[i].status));
+}
+
+/* Set LAST_COMMAND_EXIT_SIGNAL depending on STATUS.  If STATUS is -1, look
+   up PID in the pid array and set LAST_COMMAND_EXIT_SIGNAL appropriately
+   depending on its flags and exit status. */
+static int
+get_termsig (status)
+     WAIT status;
+{
+  if (WIFSTOPPED (status) == 0 && WIFSIGNALED (status))
+    return (WTERMSIG (status));
+  else
+    return (0);
+}
+
 /* Give PID the status value STATUS in the PID_LIST array. */
 static void
 set_pid_status (pid, status)
@@ -238,6 +279,8 @@ set_pid_status (pid, status)
 
   pid_list[slot].status = process_exit_status (status);
   pid_list[slot].flags &= ~PROC_RUNNING;
+  if (WIFSIGNALED (status))
+    pid_list[slot].flags |= PROC_SIGNALED;
   /* If it's not a background process, mark it as notified so it gets
      cleaned up. */
   if ((pid_list[slot].flags & PROC_ASYNC) == 0)
@@ -274,6 +317,20 @@ unset_pid_flags (pid, flags)
   pid_list[slot].flags &= ~flags;
 }
 
+/* Return the flags corresponding to PID in the PID_LIST array. */
+static int
+get_pid_flags (pid)
+     pid_t pid;
+{
+  int slot;
+
+  slot = find_index_by_pid (pid);
+  if (slot == NO_PID)
+    return 0;
+
+  return (pid_list[slot].flags);
+}
+
 static void
 add_pid (pid, async)
      pid_t pid;
@@ -306,7 +363,12 @@ mark_dead_jobs_as_notified (force)
 	ndead++;
     }
 
-  if (force == 0 && ndead <= CHILD_MAX)
+  if (child_max < 0)
+    child_max = getmaxchild ();
+  if (child_max < 0)
+    child_max = DEFAULT_CHILD_MAX;
+
+  if (force == 0 && ndead <= child_max)
     return;
 
   /* If FORCE == 0, we just mark as many non-running async jobs as notified
@@ -319,7 +381,7 @@ mark_dead_jobs_as_notified (force)
 	   pid_list[i].pid != last_asynchronous_pid)
 	{
 	  pid_list[i].flags |= PROC_NOTIFIED;
-	  if (force == 0 && (pid_list[i].flags & PROC_ASYNC) && --ndead <= CHILD_MAX)
+	  if (force == 0 && (pid_list[i].flags & PROC_ASYNC) && --ndead <= child_max)
 	    break;
 	}
     }
@@ -438,15 +500,33 @@ initialize_job_signals ()
 static void
 reap_zombie_children ()
 {
-#if defined (WNOHANG)
+#  if defined (WNOHANG)
   pid_t pid;
   WAIT status;
 
   while ((pid = waitpid (-1, (int *)&status, WNOHANG)) > 0)
     set_pid_status (pid, status);
-#endif
+#  endif /* WNOHANG */
 }
-#endif /* WAITPID && WNOHANG */
+#endif /* WAITPID */
+
+#if !defined (HAVE_SIGINTERRUPT) && defined (HAVE_POSIX_SIGNALS)
+static int
+siginterrupt (sig, flag)
+     int sig, flag;
+{
+  struct sigaction act;
+
+  sigaction (sig, (struct sigaction *)NULL, &act);
+
+  if (flag)
+    act.sa_flags &= ~SA_RESTART;
+  else
+    act.sa_flags |= SA_RESTART;
+
+  return (sigaction (sig, &act, (struct sigaction *)NULL));
+}
+#endif /* !HAVE_SIGINTERRUPT && HAVE_POSIX_SIGNALS */
 
 /* Fork, handling errors.  Returns the pid of the newly made child, or 0.
    COMMAND is just for remembering the name of the command; we don't do
@@ -559,18 +639,22 @@ wait_for_single_pid (pid)
 {
   pid_t got_pid;
   WAIT status;
-  int pstatus;
+  int pstatus, flags;
 
   pstatus = find_status_by_pid (pid);
 
   if (pstatus == PROC_BAD)
     {
-      internal_error ("wait: pid %ld is not a child of this shell", (long)pid);
+      internal_error (_("wait: pid %ld is not a child of this shell"), (long)pid);
       return (127);
     }
 
   if (pstatus != PROC_STILL_ALIVE)
-    return (pstatus);
+    {
+      if (pstatus > 128)
+	last_command_exit_signal = find_termsig_by_pid (pid);
+      return (pstatus);
+    }
 
   siginterrupt (SIGINT, 1);
   while ((got_pid = WAITPID (pid, &status, 0)) != pid)
@@ -588,13 +672,16 @@ wait_for_single_pid (pid)
 	set_pid_status (got_pid, status);
     }
 
-  set_pid_status (got_pid, status);
-  set_pid_flags (got_pid, PROC_NOTIFIED);
+  if (got_pid > 0)
+    {
+      set_pid_status (got_pid, status);
+      set_pid_flags (got_pid, PROC_NOTIFIED);
+    }
 
   siginterrupt (SIGINT, 0);
   QUIT;
 
-  return (process_exit_status (status));
+  return (got_pid > 0 ? process_exit_status (status) : -1);
 }
 
 /* Wait for all of the shell's children to exit.  Called by the `wait'
@@ -662,6 +749,7 @@ wait_sigint_handler (sig)
       restore_sigint_handler ();
       interrupt_immediately = 0;
       trap_handler (SIGINT);	/* set pending_traps[SIGINT] */
+      wait_signal_received = SIGINT;
       longjmp (wait_intr_buf, 1);
     }
 
@@ -678,6 +766,22 @@ wait_sigint_handler (sig)
   SIGRETURN (0);
 }
 
+static char *
+j_strsignal (s)
+     int s;
+{
+  static char retcode_name_buffer[64] = { '\0' };
+  char *x;
+
+  x = strsignal (s);
+  if (x == 0)
+    {
+      x = retcode_name_buffer;
+      sprintf (x, "Signal %d", s);
+    }
+  return x;
+}
+
 /* Wait for pid (one of our children) to terminate.  This is called only
    by the execution code in execute_cmd.c. */
 int
@@ -694,7 +798,11 @@ wait_for (pid)
     return (0);
 
   if (pstatus != PROC_STILL_ALIVE)
-    return (pstatus);
+    {
+      if (pstatus > 128)
+	last_command_exit_signal = find_termsig_by_pid (pid);
+      return (pstatus);
+    }
 
   /* If we are running a script, ignore SIGINT while we're waiting for
      a child to exit.  The loop below does some of this, but not all. */
@@ -719,7 +827,8 @@ wait_for (pid)
 	set_pid_status (got_pid, status);
     }
 
-  set_pid_status (got_pid, status);
+  if (got_pid > 0)
+    set_pid_status (got_pid, status);
 
 #if defined (HAVE_WAITPID)
   if (got_pid >= 0)
@@ -751,6 +860,7 @@ wait_for (pid)
   /* Default return value. */
   /* ``a full 8 bits of status is returned'' */
   return_val = process_exit_status (status);
+  last_command_exit_signal = get_termsig (status);
 
 #if !defined (DONT_REPORT_SIGPIPE)
   if ((WIFSTOPPED (status) == 0) && WIFSIGNALED (status) &&
@@ -760,7 +870,7 @@ wait_for (pid)
 	(WTERMSIG (status) != SIGINT) && (WTERMSIG (status) != SIGPIPE))
 #endif
     {
-      fprintf (stderr, "%s", strsignal (WTERMSIG (status)));
+      fprintf (stderr, "%s", j_strsignal (WTERMSIG (status)));
       if (WIFCORED (status))
 	fprintf (stderr, " (core dumped)");
       fprintf (stderr, "\n");
