@@ -1,6 +1,6 @@
 /* redir.c -- Functions to perform input and output redirection. */
 
-/* Copyright (C) 1997 Free Software Foundation, Inc.
+/* Copyright (C) 1997-2002 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -25,7 +25,7 @@
 
 #include <stdio.h>
 #include "bashtypes.h"
-#ifndef _MINIX
+#if !defined (_MINIX) && defined (HAVE_SYS_FILE_H)
 #  include <sys/file.h>
 #endif
 #include "filecntl.h"
@@ -42,6 +42,7 @@ extern int errno;
 #endif
 
 #include "bashansi.h"
+#include "bashintl.h"
 
 #include "memalloc.h"
 #include "shell.h"
@@ -53,6 +54,8 @@ extern int errno;
 #  include "input.h"
 #endif
 
+int expanding_redir;
+
 extern int posixly_correct;
 extern REDIRECT *redirection_undo_list;
 extern REDIRECT *exec_redirection_undo_list;
@@ -63,17 +66,18 @@ static void add_exec_redirect __P((REDIRECT *));
 static int add_undo_redirect __P((int));
 static int expandable_redirection_filename __P((REDIRECT *));
 static int stdin_redirection __P((enum r_instruction, int));
-static int do_redirection_internal __P((REDIRECT *, int, int, int));
+static int do_redirection_internal __P((REDIRECT *, int));
 
 static int write_here_document __P((int, WORD_DESC *));
-static int here_document_to_fd __P((WORD_DESC *));
+static int write_here_string __P((int, WORD_DESC *));
+static int here_document_to_fd __P((WORD_DESC *, enum r_instruction));
 
 static int redir_special_open __P((int, char *, int, int, enum r_instruction));
 static int noclobber_open __P((char *, int, int, enum r_instruction));
 static int redir_open __P((char *, int, int, enum r_instruction));
 
-/* Spare redirector used when translating [N]>&WORD or [N]<&WORD to a new
-   redirection and when creating the redirection undo list. */
+/* Spare redirector used when translating [N]>&WORD[-] or [N]<&WORD[-] to
+   a new redirection and when creating the redirection undo list. */
 static REDIRECTEE rd;
 
 /* Set to errno when a here document cannot be created for some reason.
@@ -92,10 +96,26 @@ redirection_error (temp, error)
   if (temp->redirector < 0)
     /* This can happen when read_token_word encounters overflow, like in
        exec 4294967297>x */
-    filename = "file descriptor out of range";
+    filename = _("file descriptor out of range");
 #ifdef EBADF
   else if (temp->redirector >= 0 && errno == EBADF)
-    filename = allocname = itos (temp->redirector);
+    {
+      /* If we're dealing with two file descriptors, we have to guess about
+         which one is invalid; in the cases of r_{duplicating,move}_input and
+         r_{duplicating,move}_output we're here because dup2() failed. */
+      switch (temp->instruction)
+        {
+        case r_duplicating_input:
+        case r_duplicating_output:
+        case r_move_input:
+        case r_move_output:
+	  filename = allocname = itos (temp->redirectee.dest);
+	  break;
+	default:
+	  filename = allocname = itos (temp->redirector);
+	  break;
+        }
+    }
 #endif
   else if (expandable_redirection_filename (temp))
     {
@@ -118,21 +138,21 @@ redirection_error (temp, error)
   switch (error)
     {
     case AMBIGUOUS_REDIRECT:
-      internal_error ("%s: ambiguous redirect", filename);
+      internal_error (_("%s: ambiguous redirect"), filename);
       break;
 
     case NOCLOBBER_REDIRECT:
-      internal_error ("%s: cannot overwrite existing file", filename);
+      internal_error (_("%s: cannot overwrite existing file"), filename);
       break;
 
 #if defined (RESTRICTED_SHELL)
     case RESTRICTED_REDIRECT:
-      internal_error ("%s: restricted: cannot redirect output", filename);
+      internal_error (_("%s: restricted: cannot redirect output"), filename);
       break;
 #endif /* RESTRICTED_SHELL */
 
     case HEREDOC_REDIRECT:
-      internal_error ("cannot create temp file for here document: %s", strerror (heredoc_errno));
+      internal_error (_("cannot create temp file for here document: %s"), strerror (heredoc_errno));
       break;
 
     default:
@@ -143,21 +163,21 @@ redirection_error (temp, error)
   FREE (allocname);
 }
 
-/* Perform the redirections on LIST.  If FOR_REAL, then actually make
-   input and output file descriptors, otherwise just do whatever is
-   neccessary for side effecting.  INTERNAL says to remember how to
-   undo the redirections later, if non-zero.  If SET_CLEXEC is non-zero,
-   file descriptors opened in do_redirection () have their close-on-exec
-   flag set. */
+/* Perform the redirections on LIST.  If flags & RX_ACTIVE, then actually
+   make input and output file descriptors, otherwise just do whatever is
+   neccessary for side effecting.  flags & RX_UNDOABLE says to remember
+   how to undo the redirections later, if non-zero.  If flags & RX_CLEXEC
+   is non-zero, file descriptors opened in do_redirection () have their
+   close-on-exec flag set. */
 int
-do_redirections (list, for_real, internal, set_clexec)
+do_redirections (list, flags)
      REDIRECT *list;
-     int for_real, internal, set_clexec;
+     int flags;
 {
   int error;
   REDIRECT *temp;
 
-  if (internal)
+  if (flags & RX_UNDOABLE)
     {
       if (redirection_undo_list)
 	{
@@ -170,7 +190,7 @@ do_redirections (list, for_real, internal, set_clexec)
 
   for (temp = list; temp; temp = temp->next)
     {
-      error = do_redirection_internal (temp, for_real, internal, set_clexec);
+      error = do_redirection_internal (temp, flags);
       if (error)
 	{
 	  redirection_error (temp, error);
@@ -197,6 +217,8 @@ expandable_redirection_filename (redirect)
     case r_output_force:
     case r_duplicating_input_word:
     case r_duplicating_output_word:
+    case r_move_input_word:
+    case r_move_output_word:
       return 1;
 
     default:
@@ -219,7 +241,9 @@ redirection_expand (word)
     w->flags |= W_NOSPLIT;
 
   tlist1 = make_word_list (w, (WORD_LIST *)NULL);
+  expanding_redir = 1;
   tlist2 = expand_words_no_vars (tlist1);
+  expanding_redir = 0;
   dispose_words (tlist1);
 
   if (!tlist2 || tlist2->next)
@@ -235,6 +259,34 @@ redirection_expand (word)
   return (result);
 }
 
+static int
+write_here_string (fd, redirectee)
+     int fd;
+     WORD_DESC *redirectee;
+{
+  char *herestr;
+  int herelen, n, e;
+
+  herestr = expand_string_to_string (redirectee->word, 0);
+  herelen = STRLEN (herestr);
+
+  n = write (fd, herestr, herelen);
+  if (n == herelen)
+    {
+      n = write (fd, "\n", 1);
+      herelen = 1;
+    }
+  e = errno;
+  FREE (herestr);
+  if (n != herelen)
+    {
+      if (e == 0)
+	e = ENOSPC;
+      return e;
+    }
+  return 0;
+}  
+
 /* Write the text of the here document pointed to by REDIRECTEE to the file
    descriptor FD, which is already open to a temp file.  Return 0 if the
    write is successful, otherwise return errno. */
@@ -316,8 +368,9 @@ write_here_document (fd, redirectee)
    by REDIRECTEE, and return a file descriptor open for reading to the temp
    file.  Return -1 on any error, and make sure errno is set appropriately. */
 static int
-here_document_to_fd (redirectee)
+here_document_to_fd (redirectee, ri)
      WORD_DESC *redirectee;
+     enum r_instruction ri;
 {
   char *filename;
   int r, fd, fd2;
@@ -334,7 +387,8 @@ here_document_to_fd (redirectee)
   errno = r = 0;		/* XXX */
   /* write_here_document returns 0 on success, errno on failure. */
   if (redirectee->word)
-    r = write_here_document (fd, redirectee);
+    r = (ri != r_reading_string) ? write_here_document (fd, redirectee)
+				 : write_here_string (fd, redirectee);
 
   if (r)
     {
@@ -416,7 +470,7 @@ redir_special_open (spec, filename, flags, mode, ri)
 {
   int fd;
 #if !defined (HAVE_DEV_FD)
-  long lfd;
+  intmax_t lfd;
 #endif
 
   fd = -1;
@@ -452,7 +506,7 @@ redir_special_open (spec, filename, flags, mode, ri)
 #if defined (HAVE_NETWORK)
       fd = netopen (filename);
 #else
-      internal_warning ("/dev/(tcp|udp)/host/port not supported without networking");
+      internal_warning (_("/dev/(tcp|udp)/host/port not supported without networking"));
       fd = open (filename, flags, mode);
 #endif
       break;
@@ -551,18 +605,19 @@ redir_open (filename, flags, mode, ri)
 
 /* Do the specific redirection requested.  Returns errno or one of the
    special redirection errors (*_REDIRECT) in case of error, 0 on success.
-   If FOR_REAL is zero, then just do whatever is neccessary to produce the
-   appropriate side effects.   REMEMBERING, if non-zero, says to remember
-   how to undo each redirection.  If SET_CLEXEC is non-zero, then
-   we set all file descriptors > 2 that we open to be close-on-exec.  */
+   If flags & RX_ACTIVE is zero, then just do whatever is neccessary to
+   produce the appropriate side effects.   flags & RX_UNDOABLE, if non-zero,
+   says to remember how to undo each redirection.  If flags & RX_CLEXEC is
+   non-zero, then we set all file descriptors > 2 that we open to be
+   close-on-exec.  */
 static int
-do_redirection_internal (redirect, for_real, remembering, set_clexec)
+do_redirection_internal (redirect, flags)
      REDIRECT *redirect;
-     int for_real, remembering, set_clexec;
+     int flags;
 {
   WORD_DESC *redirectee;
   int redir_fd, fd, redirector, r, oflags;
-  long lfd;
+  intmax_t lfd;
   char *redirectee_word;
   enum r_instruction ri;
   REDIRECT *new_redirect;
@@ -572,12 +627,14 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
   redirector = redirect->redirector;
   ri = redirect->instruction;
 
-  if (ri == r_duplicating_input_word || ri == r_duplicating_output_word)
+  if (TRANSLATE_REDIRECT (ri))
     {
-      /* We have [N]>&WORD or [N]<&WORD.  Expand WORD, then translate
+      /* We have [N]>&WORD[-] or [N]<&WORD[-].  Expand WORD, then translate
 	 the redirection into a new one and continue. */
       redirectee_word = redirection_expand (redirectee);
 
+      /* XXX - what to do with [N]<&$w- where w is unset or null?  ksh93
+	       closes N. */
       if (redirectee_word == 0)
 	return (AMBIGUOUS_REDIRECT);
       else if (redirectee_word[0] == '-' && redirectee_word[1] == '\0')
@@ -591,11 +648,21 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	    rd.dest = lfd;
 	  else
 	    rd.dest = -1;	/* XXX */
-	  new_redirect = make_redirection (redirector,
-					   (ri == r_duplicating_input_word
-						? r_duplicating_input
-						: r_duplicating_output),
-					   rd);		
+	  switch (ri)
+	    {
+	    case r_duplicating_input_word:
+	      new_redirect = make_redirection (redirector, r_duplicating_input, rd);
+	      break;
+	    case r_duplicating_output_word:
+	      new_redirect = make_redirection (redirector, r_duplicating_output, rd);
+	      break;
+	    case r_move_input_word:
+	      new_redirect = make_redirection (redirector, r_move_input, rd);
+	      break;
+	    case r_move_output_word:
+	      new_redirect = make_redirection (redirector, r_move_output, rd);
+	      break;
+	    }
 	}
       else if (ri == r_duplicating_output_word && redirector == 1)
 	{
@@ -678,9 +745,9 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
       if (fd < 0)
 	return (errno);
 
-      if (for_real)
+      if (flags & RX_ACTIVE)
 	{
-	  if (remembering)
+	  if (flags & RX_UNDOABLE)
 	    {
 	      /* Only setup to undo it if the thing to undo is active. */
 	      if ((fd != redirector) && (fcntl (redirector, F_GETFD, 0) != -1))
@@ -714,7 +781,7 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	   * both sh and ksh leave the file descriptors open across execs.
 	   * The Posix standard mentions only the exec builtin.
 	   */
-	  if (set_clexec && (redirector > 2))
+	  if ((flags & RX_CLEXEC) && (redirector > 2))
 	    SET_CLOSE_ON_EXEC (redirector);
 	}
 
@@ -732,9 +799,9 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	 redirection here. */
       if (ri == r_err_and_out)
 	{
-	  if (for_real)
+	  if (flags & RX_ACTIVE)
 	    {
-	      if (remembering)
+	      if (flags & RX_UNDOABLE)
 		add_undo_redirect (2);
 	      if (dup2 (1, 2) < 0)
 		return (errno);
@@ -744,11 +811,12 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 
     case r_reading_until:
     case r_deblank_reading_until:
+    case r_reading_string:
       /* REDIRECTEE is a pointer to a WORD_DESC containing the text of
 	 the new input.  Place it in a temporary file. */
       if (redirectee)
 	{
-	  fd = here_document_to_fd (redirectee);
+	  fd = here_document_to_fd (redirectee, ri);
 
 	  if (fd < 0)
 	    {
@@ -756,9 +824,9 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	      return (HEREDOC_REDIRECT);
 	    }
 
-	  if (for_real)
+	  if (flags & RX_ACTIVE)
 	    {
-	      if (remembering)
+	      if (flags & RX_UNDOABLE)
 	        {
 		  /* Only setup to undo it if the thing to undo is active. */
 		  if ((fd != redirector) && (fcntl (redirector, F_GETFD, 0) != -1))
@@ -781,7 +849,7 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	      duplicate_buffered_stream (fd, redirector);
 #endif
 
-	      if (set_clexec && (redirector > 2))
+	      if ((flags & RX_CLEXEC) && (redirector > 2))
 		SET_CLOSE_ON_EXEC (redirector);
 	    }
 
@@ -796,9 +864,11 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 
     case r_duplicating_input:
     case r_duplicating_output:
-      if (for_real && (redir_fd != redirector))
+    case r_move_input:
+    case r_move_output:
+      if ((flags & RX_ACTIVE) && (redir_fd != redirector))
 	{
-	  if (remembering)
+	  if (flags & RX_UNDOABLE)
 	    {
 	      /* Only setup to undo it if the thing to undo is active. */
 	      if (fcntl (redirector, F_GETFD, 0) != -1)
@@ -815,7 +885,7 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	    return (errno);
 
 #if defined (BUFFERED_INPUT)
-	  if (ri == r_duplicating_input)
+	  if (ri == r_duplicating_input || ri == r_move_input)
 	    duplicate_buffered_stream (redir_fd, redirector);
 #endif /* BUFFERED_INPUT */
 
@@ -826,16 +896,20 @@ do_redirection_internal (redirect, for_real, remembering, set_clexec)
 	     unless closed by something like `exec 2<&-'. */
 	  /* if ((already_set || set_unconditionally) && (ok_to_set))
 		set_it () */
-	  if (((fcntl (redir_fd, F_GETFD, 0) == 1) || set_clexec) &&
+	  if (((fcntl (redir_fd, F_GETFD, 0) == 1) || (flags & RX_CLEXEC)) &&
 	       (redirector > 2))
 	    SET_CLOSE_ON_EXEC (redirector);
+
+	  /* dup-and-close redirection */
+	  if (ri == r_move_input || ri == r_move_output)
+	    close (redir_fd);
 	}
       break;
 
     case r_close_this:
-      if (for_real)
+      if (flags & RX_ACTIVE)
 	{
-	  if (remembering && (fcntl (redirector, F_GETFD, 0) != -1))
+	  if ((flags & RX_UNDOABLE) && (fcntl (redirector, F_GETFD, 0) != -1))
 	    add_undo_redirect (redirector);
 
 #if defined (BUFFERED_INPUT)
@@ -872,7 +946,7 @@ add_undo_redirect (fd)
 
   if (new_fd < 0)
     {
-      sys_error ("redirection error: cannot duplicate fd");
+      sys_error (_("redirection error: cannot duplicate fd"));
       return (-1);
     }
 
@@ -946,6 +1020,7 @@ stdin_redirection (ri, redirector)
     case r_input_output:
     case r_reading_until:
     case r_deblank_reading_until:
+    case r_reading_string:
       return (1);
     case r_duplicating_input:
     case r_duplicating_input_word:
