@@ -23,9 +23,14 @@
 
 #include "stdc.h"
 
+/* Values for flags argument to do_redirections */
+#define RX_ACTIVE	0x01	/* do it; don't just go through the motions */
+#define RX_UNDOABLE	0x02	/* make a list to undo these redirections */
+#define RX_CLEXEC	0x04	/* set close-on-exec for opened fds > 2 */
+
 extern void redirection_error __P((REDIRECT *, int));
-extern int do_redirections __P((REDIRECT *, int, int, int));
+extern int do_redirections __P((REDIRECT *, int));
 extern char *redirection_expand __P((WORD_DESC *));
-extern int stdin_redirs __P((REDIRECT *));
+extern int stdin_redirects __P((REDIRECT *));
 
 #endif /* _REDIR_H_ */
