@@ -33,37 +33,60 @@
 #endif /* !NSIG */
 
 #define NO_SIG -1
-#define DEFAULT_SIG SIG_DFL
-#define IGNORE_SIG  SIG_IGN
+#define DEFAULT_SIG	SIG_DFL
+#define IGNORE_SIG	SIG_IGN
 
 /* Special shell trap names. */
-#define DEBUG_TRAP NSIG
-#define EXIT_TRAP 0
+#define DEBUG_TRAP	NSIG
+#define ERROR_TRAP	NSIG+1
+#define RETURN_TRAP	NSIG+2
+#define EXIT_TRAP 	0
 
-#define signal_object_p(x) (decode_signal (x) != NO_SIG)
+/* system signals plus special bash traps */
+#define BASH_NSIG	NSIG+3
+
+/* Flags values for decode_signal() */
+#define DSIG_SIGPREFIX	0x01		/* don't alllow `SIG' PREFIX */
+#define DSIG_NOCASE	0x02		/* case-insensitive comparison */
+
+#define signal_object_p(x,f) (decode_signal (x,f) != NO_SIG)
+
+#define TRAP_STRING(s) \
+  (signal_is_trapped (s) && signal_is_ignored (s) == 0) ? trap_list[s] \
+							: (char *)NULL
 
 extern char *trap_list[];
 
 /* Externally-visible functions declared in trap.c. */
 extern void initialize_traps __P((void));
+
 extern void run_pending_traps __P((void));
+
 extern void maybe_set_sigchld_trap __P((char *));
 extern void set_sigchld_trap __P((char *));
+
 extern void set_debug_trap __P((char *));
+extern void set_error_trap __P((char *));
+extern void set_return_trap __P((char *));
+
 extern void set_sigint_trap __P((char *));
 extern void set_signal __P((int, char *));
+
 extern void restore_default_signal __P((int));
 extern void ignore_signal __P((int));
 extern int run_exit_trap __P((void));
 extern void run_trap_cleanup __P((int));
-extern void run_debug_trap __P((void));
+extern int run_debug_trap __P((void));
+extern void run_error_trap __P((void));
+extern void run_return_trap __P((void));
+
 extern void free_trap_strings __P((void));
 extern void reset_signal_handlers __P((void));
 extern void restore_original_signals __P((void));
 
 extern char *signal_name __P((int));
 
-extern int decode_signal __P((char *));
+extern int decode_signal __P((char *, int));
 extern void run_interrupt_trap __P((void));
 extern int maybe_call_trap_handler __P((int));
 extern int signal_is_trapped __P((int));
